# Приняты некорректные данные от сокета
class IncorrectDataReceivedError(Exception):
    """
    Класс - исключение для обработки получения некорректных данных.
    """

    def __str__(self):
        return 'Принято некорректное сообщение от удаленного компьютера.'


# Аргументом функции не является словарь
class NonDictInputError(Exception):
    """
    Класс - исключение для обработки сообщения, не содержащего словарь.
    """

    def __str__(self):
        return 'Аргумент функции должен быть словарем.'


# В словаре отсутствуют обязательные поля
class ReqFieldMissingError(Exception):
    """
    Класс - исключение для обработки словарей с отсутствием необходимых полей.
    При генерации требует строку с полем, отсутствующим в словаре.
    """

    def __init__(self, missing_field):
        self.missing_field = missing_field

    def __str__(self):
        return f'В принятом словаре отсутствует обязательное поле {self.missing_field}'


# Ошибка сервера
class ServerError(Exception):
    """
    Класс - исключение для обработки ошибок сервера.
    При генерации требует строку с описанием ошибки,
    полученную с сервера.
    """

    def __init__(self, text):
        self.text = text

    def __str__(self):
        return self.text



import json
import unittest
import os
import sys

sys.path.append(os.path.join(os.getcwd(), '..'))
from common.utils import get_message, send_message
from common.variables import ACTION, PRESENCE, TIME, USER, ACCOUNT_NAME, RESPONSE, ERROR, ENCODING, \
    RESPONDEFAULT_IP_ADDRESSES


class TestSocket:
    def __init__(self, test_dict):
        self.test_dict = test_dict
        self.encoded_message = None
        self.received_message = None

    def send(self, message_to_send):
        json_test_message = json.dumps(self.test_dict)
        self.encoded_message = json_test_message.encode(ENCODING)
        self.received_message = message_to_send

    def recv(self, max_length):
        json_test_message = json.dumps(self.test_dict)
        return json_test_message.encode(ENCODING)


class TestUtils(unittest.TestCase):

    send_dict = {
        ACTION: PRESENCE,
        TIME: 2.5,
        USER: {
            ACCOUNT_NAME: 'Guest'
        }
    }

    recv_err_dict = {
        RESPONDEFAULT_IP_ADDRESSES: 400,
        ERROR: 'Bad Request'
    }

    recv_ok_dict = {
        RESPONSE: 200,
    }

    def test_get_message_correct_input(self):
        correct_test_socket = TestSocket(self.recv_ok_dict)
        correct_message = get_message(correct_test_socket)
        self.assertEqual(correct_message, self.recv_ok_dict)

    def test_get_message_incorrect_input(self):
        incorrect_test_socket = TestSocket(self.recv_err_dict)
        incorrect_message = get_message(incorrect_test_socket)
        self.assertEqual(incorrect_message, self.recv_err_dict)

    def test_get_message_wrong_dict(self):
        recv_wrong_dict = {
            RESPONSE: 200,
            'strange_stuff': {'I', 'am', 'set'}
        }
        wrong_client = TestSocket(recv_wrong_dict)

        with self.assertRaises(TypeError):
            get_message(wrong_client)

    def test_send_message_input(self):
        correct_test_socket = TestSocket(self.send_dict)
        send_message(correct_test_socket, self.send_dict)
        self.assertEqual(correct_test_socket.encoded_message, correct_test_socket.received_message)



if __name__ == '__main__':
    unittest.main()



import unittest
import os
import sys

sys.path.append(os.path.join(os.getcwd(), '..'))
from client import create_presence, process_ans
from common.variables import ACTION, PRESENCE, TIME, USER, ACCOUNT_NAME, RESPONSE, ERROR


class TestClient(unittest.TestCase):

    def test_presence_input_name(self):
        test_dict = create_presence('Alberto')
        test_dict[TIME] = 2.9
        self.assertNotEqual(test_dict, {ACTION: PRESENCE, TIME: 2.9, USER: {ACCOUNT_NAME: 'Guest'}})

    def test_presence_input_name_with_not(self):
        test_dict = create_presence('Alberto')
        test_dict[TIME] = 2.9
        self.assertNotEqual(test_dict, {ACTION: PRESENCE, TIME: 2.9, USER: {ACCOUNT_NAME: 'Guest'}})

    def test_presence_base_arg(self):
        test_dict = create_presence()
        test_dict[TIME] = 3.1
        self.assertEqual(test_dict, {ACTION: PRESENCE, TIME: 3.1, USER: {ACCOUNT_NAME: 'Guest'}})

    def test_process_ans_correct_input(self):
        response = process_ans({RESPONSE: 200})
        self.assertEqual(response, '200 : OK')

    def test_process_ans_incorrect_input_with_other_message(self):
        inc_response = process_ans({RESPONSE: 222, ERROR: 'Oh, wrong data!'})
        self.assertEqual(inc_response, '400 : Oh, wrong data!')

    def test_process_ans_without_response(self):
        with self.assertRaises(ValueError):
            process_ans({ERROR: 'I am error'})


if __name__ == '__main__':
    unittest.main()



from sqlalchemy import create_engine, Column, Text, Integer, String, DateTime, ForeignKey
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import sessionmaker
import datetime


class ServerDataBase:
    '''
    Класс - оболочка для работы с базой данных сервера.
    Использует SQLite базу данных, реализован с помощью
    SQLAlchemy ORM. Используется классический подход.
    '''
    Base = declarative_base()

    class AllUsers(Base):
        '''Класс - отображение таблицы всех пользователей.'''
        __tablename__ = 'all_users'
        id = Column(Integer, primary_key=True)
        login = Column(String, unique=True)
        last_login = Column(DateTime)
        passwd_hash = Column(String)
        pubkey = Column(Text)

        def __init__(self, login, passwd_hash):
            self.login = login
            self.passwd_hash = passwd_hash
            self.last_login = datetime.datetime.now()

    class ActiveUsers(Base):
        '''Класс - отображение таблицы активных пользователей.'''
        __tablename__ = 'active_users'
        id = Column(Integer, primary_key=True)
        user = Column(Integer, ForeignKey('all_users.id'), unique=True)
        ip = Column(String)
        port = Column(Integer)
        connection_time = Column(DateTime)

        def __init__(self, user, ip, port, connection_time):
            self.user = user
            self.ip = ip
            self.port = port
            self.connection_time = connection_time

    class LoginHistory(Base):
        '''Класс - отображение таблицы истории входов.'''
        __tablename__ = 'login_history'
        id = Column(Integer, primary_key=True)
        user = Column(Integer, ForeignKey('all_users.id'))
        ip = Column(String)
        port = Column(Integer)
        connection_time = Column(DateTime)

        def __init__(self, user, ip, port, connection_time):
            self.user = user
            self.ip = ip
            self.port = port
            self.connection_time = connection_time

    class UserContacts(Base):
        '''Класс - отображение таблицы контактов пользователей.'''
        __tablename__ = 'user_contacts'
        id = Column(Integer, primary_key=True)
        user = Column(Integer, ForeignKey('all_users.id'))
        contact = Column(Integer, ForeignKey('all_users.id'))

        def __init__(self, user, contact):
            self.user = user
            self.contact = contact

    class UserHistory(Base):
        '''Класс - отображение таблицы истории действий.'''
        __tablename__ = 'user_history'
        id = Column(Integer, primary_key=True)
        user = Column(Integer, ForeignKey('all_users.id'))
        sent = Column(Integer)
        accepted = Column(Integer)

        def __init__(self, user):
            self.user = user
            self.sent = 0
            self.accepted = 0

    def __init__(self, path):
        print(path)
        self.engine = create_engine(f'sqlite:///{path}', echo=False, pool_recycle=7200,
                                    connect_args={'check_same_thread': False})

        self.Base.metadata.create_all(self.engine)
        Session = sessionmaker(bind=self.engine)
        self.session = Session()

        self.session.query(self.ActiveUsers).delete()
        self.session.commit()

    def user_login(self, username, ip_address, port, key):
        '''
        Метод, выполняющийся при входе пользователя, записывает в базу факт входа,
        обновляет открытый ключ пользователя при его изменении.
        '''
        res = self.session.query(self.AllUsers).filter_by(login=username)

        if res.count():
            user = res.first()
            user.last_login = datetime.datetime.now()
            if user.pubkey != key:
                user.pubkey = key
        else:
            raise ValueError('Пользователь не зарегистрирован.')

        new_active_user = self.ActiveUsers(
            user.id, ip_address, port, datetime.datetime.now())
        self.session.add(new_active_user)

        history = self.LoginHistory(
            user.id, ip_address, port, datetime.datetime.now())
        self.session.add(history)

        self.session.commit()

    def user_logout(self, username):
        '''Метод фиксирующий отключения пользователя.'''
        user = self.session.query(self.AllUsers).filter_by(login=username).first()
        self.session.query(self.ActiveUsers).filter_by(user=user.id).delete()
        self.session.commit()

    def process_message(self, sender, receiver):
        '''Метод записывающий в таблицу статистики факт передачи сообщения.'''
        sender = self.session.query(self.AllUsers).filter_by(login=sender).first().id
        receiver = self.session.query(self.AllUsers).filter_by(login=receiver).first().id

        sender_row = self.session.query(self.UserHistory).filter_by(user=sender).first()
        sender_row.sent += 1

        receiver_row = self.session.query(self.UserHistory).filter_by(user=receiver).first()
        receiver_row.accepted += 1

        self.session.commit()

    def add_user(self, name, passwd_hash):
        '''
        Метод регистрации пользователя.
        Принимает имя и хэш пароля, создаёт запись в таблице статистики.
        '''
        user_row = self.AllUsers(name, passwd_hash)
        self.session.add(user_row)
        self.session.commit()
        history_row = self.UserHistory(user_row.id)
        self.session.add(history_row)
        self.session.commit()

    def remove_user(self, name):
        '''Метод, удаляющий пользователя из базы.'''
        user = self.session.query(self.AllUsers).filter_by(login=name).first()
        self.session.query(self.ActiveUsers).filter_by(login=user.id).delete()
        self.session.query(self.LoginHistory).filter_by(user=user.id).delete()
        self.session.query(self.UserContacts).filter_by(user=user.id).delete()
        self.session.query(self.UserContacts).filter_by(contact=user.id).delete()
        self.session.query(self.UserHistory).filter_by(user=user.id).delete()
        self.session.query(self.AllUsers).filter_by(login=name).delete()
        self.session.commit()

    def get_hash(self, name):
        '''Метод получения хэша пароля пользователя.'''
        user = self.session.query(self.AllUsers).filter_by(login=name).first()
        return user.passwd_hash

    def get_pubkey(self, name):
        '''Метод получения публичного ключа пользователя.'''
        user = self.session.query(self.AllUsers).filter_by(login=name).first()
        return user.pubkey

    def check_user(self, name):
        '''Метод, проверяющий существование пользователя.'''
        if self.session.query(self.AllUsers).filter_by(login=name).count():
            return True
        else:
            return False

    def add_contact(self, user, contact):
        '''Метод добавления контакта для пользователя.'''
        user = self.session.query(self.AllUsers).filter_by(login=user).first()
        contact = self.session.query(self.AllUsers).filter_by(login=contact).first()

        if not contact or self.session.query(self.UserContacts).filter_by(user=user.id, contact=contact.id).count():
            return

        new_contact = self.UserContacts(user.id, contact.id)
        self.session.add(new_contact)
        self.session.commit()

    def remove_contact(self, user, contact):
        '''Метод удаления контакта пользователя.'''
        user = self.session.query(self.AllUsers).filter_by(login=user).first()
        contact = self.session.query(self.AllUsers).filter_by(login=contact).first()

        if not contact:
            return
        self.session.query(self.UserContacts).filter_by(user=user.id, contact=contact.id).delete()
        self.session.commit()

    def users_list(self):
        '''
        Метод, возвращающий список известных пользователей
        со временем последнего входа.
        '''
        query = self.session.query(
            self.AllUsers.login,
            self.AllUsers.last_login
        )

        return query.all()

    def active_users_list(self):
        '''Метод, возвращающий список активных пользователей.'''
        query = self.session.query(
            self.AllUsers.login,
            self.ActiveUsers.ip,
            self.ActiveUsers.port,
            self.ActiveUsers.connection_time
        ).join(self.AllUsers)

        return query.all()

    def login_history(self, username=None):
        '''Метод, возвращающий историю входов.'''
        query = self.session.query(
            self.AllUsers.login,
            self.LoginHistory.ip,
            self.LoginHistory.port,
            self.LoginHistory.connection_time
        ).join(self.AllUsers)

        if username:
            query = query.filter(self.AllUsers.login == username)
        return query.all()

    def get_contacts(self, user):
        '''Метод, возвращающий список контактов пользователя.'''
        user = self.session.query(self.AllUsers).filter_by(login=user).one()
        query = self.session.query(self.UserContacts, self.AllUsers.login).filter_by(user=user.id).\
            join(self.AllUsers, self.UserContacts.contact == self.AllUsers.id)
        return [contact[1] for contact in query.all()]

    def message_history(self):
        '''Метод, возвращающий историю отправленных сообщений.'''
        query = self.session.query(
            self.AllUsers.login,
            self.AllUsers.last_login,
            self.UserHistory.sent,
            self.UserHistory.accepted
        ).join(self.AllUsers)
        return query.all()


if __name__ == '__main__':
    test_db = ServerDataBase()
    test_db.user_login('test1', '192.168.1.113', 8080)
    test_db.user_login('test2', '192.168.1.113', 8081)
    print(test_db.users_list())
    print(test_db.active_users_list())
    test_db.user_logout('test1')
    print(test_db.login_history('test2'))
    test_db.add_contact('test2', 'test1')
    test_db.add_contact('test2', 'Alice')
    test_db.add_contact('test2', 'Lila')
    test_db.remove_contact('test2', 'Lila')
    test_db.process_message('test2', 'test1')
    print(test_db.message_history())



import unittest
import os
import sys

sys.path.append(os.path.join(os.getcwd(), '..'))
from server import process_client_message
from common.variables import ACTION, PRESENCE, TIME, USER, ACCOUNT_NAME, RESPONSE, ERROR, RESPONDEFAULT_IP_ADDRESSES


class TestServer(unittest.TestCase):

    err_dict = {
        RESPONDEFAULT_IP_ADDRESSES: 400,
        ERROR: 'Bad Request'
    }

    ok_dict = {
        RESPONSE: 200,
    }

    def test_process_client_message_correct_input(self):
        correct_message = {
            ACTION: PRESENCE,
            TIME: 2.5,
            USER: {
                ACCOUNT_NAME: 'Guest'
            }
        }

        response_dict = process_client_message(correct_message)
        self.assertEqual(response_dict, self.ok_dict)

    def test_process_client_message_no_action(self):
        no_action_message = {
            TIME: 2.5,
            USER: {
                ACCOUNT_NAME: 'Guest'
            }
        }

        response_dict = process_client_message(no_action_message)
        self.assertEqual(response_dict, self.err_dict)

    def test_process_client_message_wrong_action(self):
        inc_action_message = {
            ACTION: 'run',
            TIME: 2.5,
            USER: {
                ACCOUNT_NAME: 'Guest'
            }
        }

        response_dict = process_client_message(inc_action_message)
        self.assertEqual(response_dict, self.err_dict)

    def test_process_client_message_no_time(self):
        no_time_message = {
            ACTION: 'run',
            TIME: 2.5,
            USER: {
                ACCOUNT_NAME: 'Guest'
            }
        }

        response_dict = process_client_message(no_time_message)
        self.assertEqual(response_dict, self.err_dict)

    def test_process_client_message_no_user(self):
        no_user_message = {
            ACTION: PRESENCE,
            TIME: 2.5,
        }

        response_dict = process_client_message(no_user_message)
        self.assertEqual(response_dict, self.err_dict)

    def test_process_client_message_wrong_name(self):
        inc_name_message = {
            ACTION: PRESENCE,
            TIME: 2.5,
            USER: {
                ACCOUNT_NAME: 'James'
            }
        }

        response_dict = process_client_message(inc_name_message)
        self.assertEqual(response_dict, self.err_dict)


if __name__ == '__main__':
    unittest.main()



import sys
import os
import logging
import logging.handlers

sys.path.append('../')

# Настраиваем директорию для вывода логов
path = os.path.dirname(os.path.abspath(__file__))
path = os.path.join(path, 'client.log')

# Создаем логгер
client_log = logging.getLogger('client')

# Создаем шаблон выводимого сообщения
server_formatter = logging.Formatter('%(asctime)s %(levelname)-10s %(filename)s %(message)s')

# Создаем файл для вывода сообщений (с ротацией)
file_handler = logging.FileHandler(path, encoding='utf-8')
file_handler.setLevel(logging.DEBUG)
file_handler.setFormatter(server_formatter)

client_log.addHandler(file_handler)
client_log.setLevel(logging.DEBUG)


if __name__ == '__main__':
    client_log.debug('Отладочное сообщение')
    client_log.info('Информационное сообщение')
    client_log.warning('Предупреждение')
    client_log.error('Ошибка')
    client_log.critical('Критическая ошибка')



import logging
import inspect
import socket
import sys
import logging
import logs.config_server_log
import logs.config_client_log

if sys.argv[0].find('client') == -1:
    func_logger = logging.getLogger('server')
else:
    func_logger = logging.getLogger('client')


def log(func):
    """
    Декоратор, выполняющий логирование вызовов функций.
    Сохраняет события типа debug, содержащие
    информацию об имени вызываемой функиции, параметры, с которыми
    вызывается функция, и модуль, вызывающий функцию.
    """
    def wrapper(*args, **kwargs):
        func_result = func(*args, **kwargs)
        func_name = func.__name__
        func_module = func.__module__
        if args or kwargs:
            param_text = f'с параметрами {args}, {kwargs} '
        else:
            param_text = ''
        text_to_log = f'Функция {func_name} {param_text}' \
                      f'из модуля {func_module} была вызвана из функции {inspect.stack()[1][3]}'

        func_logger.debug(text_to_log)
        return func_result
    return wrapper



from setuptools import setup, find_packages

setup(name="megaLexa_server_messenger",
      version="1.1.0",
      description="megaLexa_server_messenger",
      author="MegaLexa",
      author_email="itsnotmyrealmail@mail.ru",
      packages=find_packages(),
      install_requires=['PyQt5', 'sqlalchemy', 'pycryptodome', 'pycryptodomex']
      )



import dis


# Метакласс для проверки сервера:
class ServerVerifier(type):
    '''
    Метакласс, проверяющий, что в результирующем классе нет клиентских
    вызовов, таких как: connect. Также проверяется, что серверный
    сокет является TCP и работает по IPv4 протоколу.
    '''

    def __init__(cls, cls_name, bases, cls_dict):
        methods = []
        attrs = []
        for func in cls_dict:
            try:
                ret = dis.get_instructions(cls_dict[func])
            except TypeError:
                pass
            else:
                for i in ret:
                    # print(i)
                    if i.opname == 'LOAD_GLOBAL':
                        if i.argval not in methods:
                            methods.append(i.argval)
                    elif i.opname == 'LOAD_ATTR':
                        if i.argval not in attrs:
                            attrs.append(i.argval)
        # print(methods)
        # print(attrs)

        if 'connect' in methods:
            raise TypeError('Исползование метода connect недопустимо в серверном классе')
        if not ('SOCK_STREAM' in attrs and 'AF_INET' in attrs):
            raise TypeError('Некорректная инициализация сокета.')
        super().__init__(cls_name, bases, cls_dict)


# Метакласс для проверки клиентов
class ClientVerifier(type):
    '''
    Метакласс, проверяющий, что в результирующем классе нет серверных
    вызовов таких как: accept, listen. Также проверяется, что сокет не
    создаётся внутри конструктора класса.
    '''

    def __init__(cls, cls_name, bases, cls_dict):
        methods = []
        for func in cls_dict:
            try:
                ret = dis.get_instructions(cls_dict[func])
            except TypeError:
                pass
            else:
                for i in ret:
                    # print(i)
                    if i.opname == 'LOAD_GLOBAL':
                        if i.argval not in methods:
                            methods.append(i.argval)
        # print(methods)

        for command in ('accept', 'listen', 'socket'):
            if command in methods:
                raise TypeError('В классе обнаружено использование запрещенного метода')
        if 'get_message' in methods or 'send_message' in methods:
            pass
        else:
            raise TypeError('Отсутствуют вызовы функций, работающих с сокетами.')
        super().__init__(cls_name, bases, cls_dict)
